@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from contextlib import ExitStack
 from dataclasses import dataclass, field
 from queue import SimpleQueue
@@ -146,6 +148,7 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
                         ):
                             yield range_info.create_row()
 
+                        range_info.text_id = src_row.text_id
                         range_info.source_refs.append(src_row.ref)
                         range_info.target_refs.append(trg_row.ref)
                         source_same_ref_rows.clear()
@@ -184,6 +187,7 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
 
             while src_row is not None:
                 if not self._all_target_rows and src_row.is_in_range:
+                    range_info.text_id = src_row.text_id
                     range_info.source_refs.append(src_row.ref)
                     target_same_ref_rows.clear()
                     range_info.source_segment.extend(src_row.segment)
@@ -196,6 +200,7 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
 
             while trg_row is not None:
                 if not self._all_source_rows and trg_row.is_in_range:
+                    range_info.text_id = trg_row.text_id
                     range_info.target_refs.append(trg_row.ref)
                     source_same_ref_rows.clear()
                     range_info.target_segment.extend(trg_row.segment)
@@ -211,14 +216,21 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
 
     def _create_rows(
         self,
-        range_info: "_RangeInfo",
+        range_info: _RangeInfo,
         src_row: Optional[TextRow],
         trg_row: Optional[TextRow],
         aligned_word_pairs: Optional[Collection[AlignedWordPair]] = None,
     ) -> Iterable[ParallelTextRow]:
         if range_info.is_in_range:
             yield range_info.create_row()
+        if src_row is not None:
+            text_id = src_row.text_id
+        elif trg_row is not None:
+            text_id = trg_row.text_id
+        else:
+            raise ValueError("Either a source or target must be specified.")
         yield ParallelTextRow(
+            text_id,
             [] if src_row is None else [src_row.ref],
             [] if trg_row is None else [trg_row.ref],
             [] if src_row is None else src_row.segment,
@@ -235,7 +247,7 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
 
     def _create_source_rows(
         self,
-        range_info: "_RangeInfo",
+        range_info: _RangeInfo,
         source_row: TextRow,
         target_same_ref_rows: List[TextRow],
     ) -> Iterable[ParallelTextRow]:
@@ -249,7 +261,7 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
 
     def _create_target_rows(
         self,
-        range_info: "_RangeInfo",
+        range_info: _RangeInfo,
         target_row: TextRow,
         source_same_ref_rows: List[TextRow],
     ) -> Iterable[ParallelTextRow]:
@@ -264,6 +276,7 @@ class StandardParallelTextCorpus(ParallelTextCorpus):
 
 @dataclass
 class _RangeInfo:
+    text_id: str = field(default="", init=False)
     source_refs: List[Any] = field(default_factory=list, init=False)
     target_refs: List[Any] = field(default_factory=list, init=False)
     source_segment: List[str] = field(default_factory=list, init=False)
@@ -279,6 +292,7 @@ class _RangeInfo:
 
     def create_row(self) -> ParallelTextRow:
         row = ParallelTextRow(
+            self.text_id,
             self.source_refs.copy(),
             self.target_refs.copy(),
             self.source_segment.copy(),
@@ -292,6 +306,7 @@ class _RangeInfo:
             is_target_range_start=False,
             is_empty=self.is_source_empty or self.is_target_empty,
         )
+        self.text_id = ""
         self.source_refs.clear()
         self.target_refs.clear()
         self.source_segment.clear()
@@ -346,7 +361,7 @@ class _TargetCorpusGenerator(ContextManager["_TargetCorpusGenerator"], Generator
         super().close()
         self._generator.close()
 
-    def __enter__(self) -> "_TargetCorpusGenerator":
+    def __enter__(self) -> _TargetCorpusGenerator:
         return self
 
     def __exit__(self, type: Any, value: Any, traceback: Any) -> None:
@@ -372,6 +387,7 @@ class _TargetCorpusGenerator(ContextManager["_TargetCorpusGenerator"], Generator
                 seg_list[range_start_offset] = (
                     range_start_verse_ref,
                     TextRow(
+                        range_start_row.text_id,
                         range_start_row.ref,
                         list(range_start_row.segment) + list(row.segment),
                         range_start_row.is_sentence_start,
@@ -380,7 +396,7 @@ class _TargetCorpusGenerator(ContextManager["_TargetCorpusGenerator"], Generator
                         is_empty=range_start_row.is_empty and row.is_empty,
                     ),
                 )
-                row = TextRow(row.ref, is_in_range=True)
+                row = TextRow(row.text_id, row.ref, is_in_range=True)
                 range_start_offset -= 1
             else:
                 range_start_offset = -1
